// Author: Alexander S. Gerasimov

#include <string>
#include "map.h"
#include "expr_interpr.h"


Variable::Variable(std::string name) : id(name) {}

std::string Variable::toString() { return id; }

int Variable::eval(const Map<std::string, int>& varToInt)
        throw(ValueNotAssignedException) {
    if (int* p = varToInt.get(id)) {
        return *p;
    } else {
        throw ValueNotAssignedException();
    }
}


Constant::Constant(int value) throw(NegativeArgumentException) {
    if (value < 0) {
        throw NegativeArgumentException();
    }
    val = value;
}

std::string Constant::toString() {
    if (0 == val) {
        return "0";
    }
    std::string s;
    int q = val;
    const int DECIMAL_BASE = 10;
    while (q > 0) {
        char digit = q % DECIMAL_BASE + '0';
        s = digit + s;
        q /= DECIMAL_BASE;
    }
    return s;
}

int Constant::eval(const Map<std::string, int>& varToInt) throw() {
    return val;
}


PlusExpr::PlusExpr(Expression* oper1, Expression* oper2)
        throw (NullPointerException) {
    if (0 == oper1 || 0 == oper2) {
        throw NullPointerException();
    }
    op1 = oper1;
    op2 = oper2;
}

PlusExpr::~PlusExpr() {
    delete op1;
    delete op2;
}

std::string PlusExpr::toString() {
    return '(' + op1->toString() + '+' + op2->toString() + ')';
}

int PlusExpr::eval(const Map<std::string, int>& varToInt)
        throw(ValueNotAssignedException) {
    return op1->eval(varToInt) + op2->eval(varToInt);
}


MultExpr::MultExpr(Expression* oper1, Expression* oper2)
        throw (NullPointerException) {
    if (0 == oper1 || 0 == oper2) {
        throw NullPointerException();
    }
    op1 = oper1;
    op2 = oper2;
}

MultExpr::~MultExpr() {
    delete op1;
    delete op2;
}

std::string MultExpr::toString() {
    return '(' + op1->toString() + '*' + op2->toString() + ')';
}

int MultExpr::eval(const Map<std::string, int>& varToInt)
        throw(ValueNotAssignedException) {
    return op1->eval(varToInt) * op2->eval(varToInt);
}
